<!DOCTYPE html>
<html>

<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title>Glances</title>

    <link rel="icon" type="image/x-icon" href="favicon.ico" />
    <script>
      window.__GLANCES__ = {
        'refresh-time': '{{ refresh_time }}'
      }
    </script>
    <script src="glances.js" defer></script>

</head>

<body>
  <div id="app"></div>
</body>
</html>
